import spacy

from entity_linker import EntityLinker
from dual_encoder import DualEncoder
from document import Document
from entity_type import EntityType
from embedding_utils import EmbeddingUtils

polyseme_test = (
    """
I have a cold and I've been sneezing all day. I feel cold, it's so cold outside. 
The union of two sets is not the intersection of those sets. I had to set down the
coffee on the table. Miles Davis played in over eight different sets. Erosion makes
strange patterns in banks. I put my money in a bank. My snowboard drifted off a bank.
There are a set of different sets that Miles Davis played in. I don't know how to write
macros in Rust. I've joined the rustacean community on rust. I like the western setting
of rust. Rust is eating away at my iron pipes.""",
    [
        EntityType("1", "common cold"),
        EntityType("2", "cold temperature"),
        EntityType("3", "set as in group"),
        EntityType("4", "set something down"),
        EntityType("5", "jazz set"),
        EntityType("6", "river bank"),
        EntityType("7", "financial bank"),
        EntityType("8", "snowbank"),
        EntityType("9", "rust programming language"),
        EntityType("10", "rust game"),
        EntityType("11", "rust oxidation"),
    ],
)

politics_test = (
    """
    That high favorability rating could translate to future support for DeSantis, especially in combination with his election pitch, which is essentially “MAGA without the mess.” We can see this in Morning Consult’s Republican primary tracker: Although DeSantis has lost ground to Trump in first-choice support, about half of Trump voters listed DeSantis as their second choice in a poll conducted May 19-21, putting him miles ahead of former Vice President Mike Pence (14 percent). If Trump falters as the primary race plays out, DeSantis could be well-positioned to capitalize and gain ground in the polls.

For all the coverage of Trump’s early endorsement edge, DeSantis also has support from GOP officeholders that could signal important on-the-ground backing in key states. For instance, more than one-third of Iowa’s Republican state legislators have endorsed DeSantis, a positive sign in the traditional lead-off state in presidential nomination contests. In December, about a quarter of the GOP membership in Michigan’s state legislature signed a letter asking DeSantis to run; while not a formal endorsement, it showed potential support for DeSantis in a pivotal battleground state. And while Trump has garnered backing from many Floridians in Congress, DeSantis has the overwhelming support of members in the state legislature, indicating that DeSantis doesn’t lack home-base support.

To top it all off, DeSantis has a sterling fundraising record, and despite the concerns of some big donors, he could enjoy ample outside support, too. During his 2022 reelection campaign, DeSantis raised more than $200 million, which OpenSecrets found was the largest ever haul by a candidate in a gubernatorial contest (not adjusted for inflation). And more than $80 million left over from that campaign could aid his presidential bid, too, albeit by transferring it to Never Back Down, the main pro-DeSantis super PAC — a move that some watchdog groups say flouts campaign finance law.

Although these forces are working in DeSantis’s favor, he still faces the challenge of weaving together a coalition of Trump fans and skeptics. On the one hand, DeSantis needs to win over more moderate or at least somewhat conservative Republicans, who aren’t as pro-Trump as more conservative voters. In fact, while Trump won the 2016 nomination thanks especially to support among somewhat conservative and moderate primary voters, he is polling better among more conservative voters this time around, though he usually holds big leads among conservatives as a whole. But DeSantis has raced to the right on some issues, like abortion, in a way that could actually alienate some less conservative Republicans. Meanwhile, Trump’s positioning as the more moderate choice by attacking DeSantis on entitlements could coax less Trumpy voters to view DeSantis as too extreme.

At the same time however, DeSantis may have an opportunity to get to Trump’s right on abortion and win over very conservative voters. Trump criticized DeSantis’s decision to sign a six-week abortion ban into law in Florida, calling it too “harsh” while resisting committing to a 15-week national ban. However, a Fabrizio Lee/Impact Research survey for the Wall Street Journal last month showed that around two-thirds of Republican primary voters back a six-week ban, and national anti-abortion leaders have signaled they may end up opposing Trump if he won’t express support for a nationwide ban. DeSantis responded to Trump, saying he “was proud” to have signed Florida’s legislation and hitting Trump for not saying whether he would have.

However DeSantis navigates his coalition challenges, he has a résumé full of conservative accomplishments to sell to primary voters and will likely have the resources to stay in the race for a long time against Trump. And, despite DeSantis’s antipathy toward mainstream media outlets, he could also benefit from the media’s desire for a competitive primary. A “DeSantis resurrection” narrative will be an extremely tempting story to tell, so any positive signs for DeSantis — a few stronger polls after his announcement, a rush of endorsements — could tee up a series of sunny stories after months of stormy headlines. Who knows? Perhaps DeSantis will even get a chance to adopt the nickname former President Bill Clinton gave himself in the 1992 Democratic presidential primary: the “Comeback Kid.”""",
    [
        EntityType("VP", "Vice President"),
        EntityType("rep", "Republican"),
        EntityType("RDS", "Ron DeSantis"),
        EntityType("DJT", "Donald J. Trump"),
        EntityType("FL", "Florida"),
    ],
)


def main():
    nlp = spacy.load("en_core_web_sm")

    (text, labels) = politics_test

    el = EntityLinker(
        DualEncoder(),
        labels,
    )

    doc = Document(nlp(text))

    el.link_entities_in_document(doc, show_progress_bar=True)

    print(doc.get_annotated_string(True))


"""
    avged_embedding = EmbeddingUtils.average_embeddings(
        [
            el.dual_encoder.encode("it"),
            el.dual_encoder.encode("of"),
            el.dual_encoder.encode("the"),
            el.dual_encoder.encode("not"),
            el.dual_encoder.encode("so"),
            el.dual_encoder.encode("could"),
        ]
    )

    print(avged_embedding)
"""


if __name__ == "__main__":
    main()
