# import copy

# import torch.nn as nn
# from modeling_bert import BertPreTrainedModel
# from transformers import PreTrainedConfig


from sentence_transformers import SentenceTransformer
from context_window import ContextWindow


class DualEncoder:
    """
    DualEncoder

    The DualEncoder is a BERT model that converts name mentions into embeddings that can be commpared.

    ------
    PARAMS:

    config: PreTrainedConfig
        The configuration object
    num_tags: int
        The numebr of tags for the model
    bert_mention: BertPreTrainedModel
        The BERT model to use for candidates
    hidden_size: int
        The hidden size of the model
    bound_classifier: nn.Linear
        The linear layer for classification
    loss_fn_linker: nn.CrossEntropyLoss
        The loss function for the linker
    loss_fn_ner: BCEWithLogitsLoss()
        The loss function for named entity recognition
    """

    """
    def __init__(self, config: PreTrainedConfig, pretrained_bert: BertPreTrainedModel):
        self.config = config
        self.num_tags = 3
        self.bert_mention = pretrained_bert
        self.bert_candidate = copy.deepcopy(pretrained_bert)
        self.hidden_size = config.hidden_size
        self.bound_classifier = nn.Linear(config.hidden_size, 3)
        self.init_modules()
        self.loss_fn_linker = nn.CrossEntropyLoss(ignore_index=-1)
        self.loss_fn_ner = nn.BCEWithLogitsLoss()
        self.BIGINT = 1e3
    """

    # The code above is for a future, purpose-built dual encoder. Below is an implementation just using a standard BERT model.

    def __init__(self):
        self.model = SentenceTransformer("all-MiniLM-L6-v2")

    def encode(self, term: str, context: ContextWindow = None):
        """
        encode

        Returns the embedding of a term and its associated context

        --------
        PARAMS:

        term: str
            The term to find the embedding of
        context: ContextWindow
            The terms' context. If omitted, only the embedding of the term in isolation is returned.
        """
        prepared_string = self._prepare_string(term, context)
        context_embedding = self.model.encode(prepared_string)
        if context is None:
            return context_embedding
        term_embedding = self.model.encode(self._prepare_string(term))
        embedding = []
        for context_component, term_component in zip(context_embedding, term_embedding):
            embedding.append((context_component + term_component) / 2)
        return embedding

    def _prepare_string(self, term: str, context: ContextWindow = None):
        """
        _prepare_string

        Performs string-cleaning tasks to prepare a term and context window for encoding and returns the prepared string

        -------
        PARAMS:

        term: str
            The term to prepare
        context: ContextWindow
            The terms' context. If omitted, only the term itself will be prepared.
        """
        if context == None:
            return term.strip()
        else:
            context_str = context.span.text.strip()
            trimmed_term = term.strip()
            prepared_string = trimmed_term + "[SEP]" + context_str
            return prepared_string
