from typing import List, Tuple

from spacy.language import Language
from spacy.tokens import Doc
from thinc.api import Config, Model, Ragged
from thinc.types import Floats2d

from crtl.components.pre_extracted_recognizer import PreExtractedRecognizer

default_config = """
[model]
@architectures = "pre_extracted_recognizer"

[model.reducer]
@layers = spacy.mean_max_reducer.v1
hidden_size = 128

[model.tok2vec]
@architectures = "spacy.Tok2Vec.v2"

[model.tok2vec.embed]
@architectures = "spacy.MultiHashEmbed.v2"
width = 96
rows = [5000, 2000, 1000, 1000]
attrs = ["ORTH", "PREFIX", "SUFFIX", "SHAPE"]
include_static_vectors = false
"""

DEFAULT_MODEL = Config().from_str(default_config)["model"]


@Language.factory(
    "pre_extracted_er",
    assigns=["token.label"],
    default_config={
        "model": DEFAULT_MODEL,
    },
    default_score_weights={"spans_sc_f": 1.0, "spans_sc_p": 0.0, "spans_sc_r": 0.0},
)
def make_pre_extracted_er(
    nlp: Language,
    name: str,
    model: Model[Tuple[List[Doc], Ragged], Floats2d],
) -> PreExtractedRecognizer:
    return PreExtractedRecognizer(nlp.vocab, model=model, name=name, ent_or_span="ENT")


@Language.factory(
    "pre_extracted_sr",
    assigns=["span.label"],
    default_config={"model": DEFAULT_MODEL, "span_key": "sc"},
)
def make_pre_extracted_sr(
    nlp: Language,
    name: str,
    model: Model[Tuple[List[Doc], Ragged], Floats2d],
    span_key: str,
) -> PreExtractedRecognizer:
    return PreExtractedRecognizer(nlp.vocab, model=model, name=name, ent_or_span="SPAN")
