import json
import pytest
from spacy.lang.en import English
from spacy.tokens import Doc, Span

nlp = English()

doc = Doc(
    nlp.vocab,
    [
        "My",
        "type",
        "2",
        "head",
        "cold",
        "has",
        "given",
        "me",
        "headaches",
        "and",
        "made",
        "me",
        "cold",
        ".",
    ],
    [
        True,
        False,
        True,
        True,
        True,
        True,
        True,
        True,
        True,
        True,
        True,
        True,
        False,
        False,
    ],
)
Span.set_extension("is_broad", default=False)
type_two_head_cold_span = Span(doc, 1, 5, kb_id="C00001")
type_two_head_cold_span._.is_broad = True
head_cold_span = Span(doc, 3, 5, kb_id="C00002")
head_cold_span._.is_broad = True
head_span = Span(doc, 3, 4, kb_id="NIL")
head_span._.is_broad = False
cold_span = Span(doc, 4, 5, kb_id="C00002")
cold_span._.is_broad = False
headaches_span = Span(doc, 8, 9, kb_id="C00001")
headaches_span._.is_broad = True
cold_temp_span = Span(doc, 12, 13, kb_id="C00003")
cold_temp_span._.is_broad = False
spans = [
    type_two_head_cold_span,
    head_cold_span,
    head_span,
    cold_span,
    headaches_span,
    cold_temp_span,
]

span_jsons = [
    {
        "start": span.start_char,
        "end": span.end_char,
        "token_start": span.start,
        "token_end": span.end,
    }
    for span in spans
]

spans_with_concept_jsons = [
    {
        "concept": "NO CONCEPT" if span.kb_id_ == "NIL" else span.kb_id_,
        "isBroad": span._.is_broad,
    }
    | span_json
    for span, span_json in zip(spans, span_jsons)
]

token_jsons = []

for i, token in enumerate(doc):
    token_span = doc[i : i + 1]
    token_json = {
        "text": token_span.text,
        "start": token_span.start_char,
        "end": token_span.end_char,
        "id": i,
        "ws": token.whitespace_ == " ",
    }
    token_jsons.append(token_json)
example_json = {
    "text": doc.text,
    "spans": span_jsons,
    "spansWithConcept": spans_with_concept_jsons,
    "tokens": token_jsons,
}
json.dump(example_json, open("new_hybrid_eg.json", "w"))
