# take the dataset defined using json and produce the document set.
from typing import Dict
from spacy.lang.en import English
from spacy.tokens import Span, Doc

nlp = English()
Span.set_extension("is_broad", default=False, force=True)


def create_doc(json_dict: Dict):
    doc = Doc(
        nlp.vocab,
        [token["text"] for token in json_dict["tokens"]],
        [token["ws"] for token in json_dict["tokens"]],
    )

    spans = [
        Span(
            doc,
            span["token_start"],
            span["token_end"] + 1,
            kb_id="NIL" if span["concept"].lower() == "no concept" else span["concept"],
        )
        for span in json_dict["spansWithConcept"]
    ]
    for span, span_dict in zip(spans, json_dict["spansWithConcept"]):
        span._.is_broad = span_dict["isBroad"]
    doc.spans["sc"] = spans
    return doc
