from flask import Flask, request
from flask_restful import Resource, Api
from flask_cors import CORS

from crtl.lkb.api.controller import create_trendnet_api_controller

controller = create_trendnet_api_controller()


class GetConceptInfo(Resource):
    def get(self):
        args = request.args
        concept_id = args["cui"]
        return controller.get_concept_info(concept_id)


class GetTermInfo(Resource):
    def get(self):
        args = request.args
        term_id = args["tui"]
        return controller.get_term_info(term_id)


class SearchForConcept(Resource):
    def get(self):
        args = request.args
        search_term = args["search_term"]
        return controller.search_for_concept(search_term)


def get_app():
    app = Flask(__name__)
    CORS(app)
    api = Api(app)
    api.add_resource(GetConceptInfo, "/get_concept_info")
    api.add_resource(GetTermInfo, "/get_term_info")
    api.add_resource(SearchForConcept, "/search_for_concept")
    return app


def start_api():
    app = get_app()
    app.run(host="0.0.0.0", port=8080, debug=False, use_reloader=False)
    print("App is running.")


if __name__ == "__main__":
    start_api()
