import pathlib
from typing import Dict, List
import json

from crtl.lkb.data_pipeline.validate_crtl_json import validate_json_content_from_files


def convert_trendnet_to_scispacy_jsonl_and_validate(input_path, output_path):
    new_concepts = convert_trendnet_to_scispacy_json(input_path)
    with open(output_path, "w") as f:
        for new_concept in new_concepts:
            json.dump(new_concept, f)
            f.write("\n")
    validate_json_content_from_files(
        output_path, pathlib.Path("test_data/TRENDNet.jsonl")
    )


def convert_trendnet_to_scispacy_json(input_path):
    with open(input_path) as f:
        old_trendnet_data = json.load(f)
    new_concepts = get_concepts(old_trendnet_data["concepts"])
    get_terms(
        old_trendnet_data["lexical_edges"], old_trendnet_data["terms"], new_concepts
    )
    get_related_concepts(
        old_trendnet_data["conceptual_edges"],
        old_trendnet_data["conceptual_relations"],
        new_concepts,
    )
    return new_concepts


def get_concepts(old_concepts: List[Dict]) -> List[Dict]:
    new_concepts = []
    for concept in old_concepts:
        new_concept = {}
        new_concept["concept_id"] = concept["id"]
        new_concept["canonical_name"] = concept["concept_name"]
        new_concept["types"] = [concept["semantic_type"]]
        new_concept["definition"] = concept["definition"]
        new_concept["terms"] = []
        new_concept["related_concepts"] = []
        new_concepts.append(new_concept)
    return new_concepts


def get_terms(old_concept_to_term_relations, old_terms, new_concepts):
    concept_id_to_concept = {concept["concept_id"]: concept for concept in new_concepts}
    term_id_to_string = {old_term["id"]: old_term["string"] for old_term in old_terms}
    for old_concept_to_term_relation in old_concept_to_term_relations:
        concept_id, term_id = (
            old_concept_to_term_relation["concept_id"],
            old_concept_to_term_relation["term_id"],
        )
        new_concept = concept_id_to_concept[concept_id]
        new_concept["terms"].append(term_id_to_string[term_id])


def get_related_concepts(
    old_concept_to_concept_relations, old_relation_names, new_concepts
):
    concept_id_to_concept = {concept["concept_id"]: concept for concept in new_concepts}
    relation_id_to_relation_name = {
        old_relation["id"]: old_relation["string"]
        for old_relation in old_relation_names
    }
    for old_concept_to_concept_relation in old_concept_to_concept_relations:
        concept_id_1, concept_id_2, rel_id = (
            old_concept_to_concept_relation["concept_id_1"],
            old_concept_to_concept_relation["concept_id_2"],
            old_concept_to_concept_relation["rel_id"],
        )
        new_concept = concept_id_to_concept[concept_id_1]
        new_concept["related_concepts"].append(
            {
                "relation": relation_id_to_relation_name[rel_id],
                "concept_id": concept_id_2,
            }
        )
