import json
import pathlib
from typing import Dict


def validate_json_content_from_files(
    actual_lkb_path: pathlib.Path, expected_lkb_path: pathlib.Path
):
    with open(expected_lkb_path, "r") as fp:  # tie this data to this python file
        expected_lkb_info = json.load(fp)
    with open(actual_lkb_path) as fp:
        lkb_info = [json.loads(line) for line in list(fp)]
    compare_test_data_to_actual_data(lkb_info, expected_lkb_info)


def compare_test_data_to_actual_data(lkb_jsonl, expected_lkb_info):
    concept_id_to_concept: Dict[str, dict] = {}
    for line in lkb_jsonl:
        concept_id_to_concept[line["concept_id"]] = line
    for expected_concept in expected_lkb_info:
        concept_in_lkb = concept_id_to_concept[expected_concept["concept_id"]]
        assert expected_concept["concept_id"] in concept_id_to_concept
        assert set(expected_concept["terms"]).issubset(set(concept_in_lkb["terms"]))
        if "types" in expected_concept:
            assert set(expected_concept["types"]) == set(concept_in_lkb["types"])
        if "definition" in expected_concept:
            assert expected_concept["definition"] == concept_in_lkb["definition"]
        if "related_concepts" in expected_concept:
            assert set(
                related_concept["concept_id"]
                for related_concept in expected_concept["related_concepts"]
            ).issubset(
                set(
                    related_concept["concept_id"]
                    for related_concept in concept_in_lkb["related_concepts"]
                )
            )
